SIDE_ENEMY = 1
SIDE_ALLY = 0

class Bullet < PhysicObject
  attr_accessor :side, :stats

  def initialize(ax, ay, avx, avy, side, stats)
    super(MAT_SHOT)
    warp(ax, ay)
    @stats = stats
    @side = side
    @ticks = stats[:maxrange]
    @body.v = vec2(avx, avy)
    @picture_id = stats[:picture]
    @did_hit = {}
  end

  def draw
    angle = -Math::PI / 2 - Math.atan2(@body.v.y, @body.v.x)
    RES::All.draw_frame(@picture_id, x, y, OBJSIZE / SPRITESIZE / 2, OBJSIZE / SPRITESIZE / 2, angle * 180 / Math::PI)
  end

  def rico
    @ticks = 10 if @ticks > 10
  end

  def finish
    @dead = true
    $app.explosions << self if stats[:explode]
  end

  def explode
    if expl_stats = stats[:explode]
      expl_stats[:count].times do
        angle = rand * 2 * Math::PI
        spd = vec2(Math.cos(angle), Math.sin(angle)).normalize * expl_stats[:speed] * (1 + rand / 10)
        $app.objects << Bullet.new(x, y, spd.x, spd.y, @side, expl_stats)
      end
    end
  end

  def update
    @ticks -= 1
    finish if @ticks <= 0
    super
  end

  def collide_walls(walls)
    rico
    true
  end

  def collide_door(door)
    # door.break if rand < 0.1
    rico
    true
  end

  def collide_player(player)
    return false if self.side == player.side
    player.take_hit(self)
    if @stats[:force]
      player.body.apply_impulse((player.body.p - @body.p).normalize * @stats[:force] * SHOOT_FORCES_SCALE, CP::ZERO_VEC_2)
    end
    finish
    true
  end

  def collide_enemy(enemy)
    return false if self.side == enemy.side
    return false if enemy.dead && rand < 0.5
    return false if @did_hit[enemy]
    if @stats[:force]
      enemy.body.apply_impulse((enemy.body.p - @body.p).normalize * @stats[:force] * SHOOT_FORCES_SCALE, CP::ZERO_VEC_2)
    end
    enemy.take_hit(self)
    if @stats[:pierce]
      @did_hit[enemy] = true
      return false
    end
    finish
    true
  end

  def dontcollide_bullet(bullet)
  end
end
