# encoding: utf-8

require_relative "./lib/nonoengine"
require_relative "./resources"
require_relative "./gameobject"
require_relative "./physics_wrap"
require_relative "./physics"
require_relative "./player"
require_relative "./map"
require_relative "./bullet"
require_relative "./enemy"

class FTPT < GameApp
  attr_accessor :physics
  attr_accessor :player
  attr_accessor :map
  attr_accessor :objects
  attr_accessor :broken_doors
  attr_accessor :explosions
  attr_accessor :time

  def spawn_enemy(template)
    enemy = Enemy.new(template)
    pt = @map.random_point(LogicalTile::Passage)
    enemy.warp(pt[0] * TILESIZE, pt[1] * TILESIZE)
    @objects << enemy
  end

  def new_player
    if @player
      @objects.delete player
      @player.die
    end
    @gamma_timer = 0
    @player = Player.new(ALL_CLASSES.sample)
    pt = @map.random_point(LogicalTile::Room) || @map.random_point(LogicalTile::Passage)
    @player.warp(pt[0] * TILESIZE, pt[1] * TILESIZE)
    @objects << @player
  end

  def initialize
    $app = self
    super
    Engine[Params::Width] = SCREENX
    Engine[Params::Height] = SCREENY
    Engine::init "resources"

    @physics = PhysicEngine.new
    @physics.register Player
    @physics.register Walls
    @physics.register Door
    @physics.register Bullet
    @physics.register Enemy
    @physics.end_registration

    @map = Map.new
    @objects << @map

    new_player

    10.times do
      spawn_enemy(ENEMY_ARCHER)
    end
    10.times do
      spawn_enemy(ENEMY_MELEE)
    end
    10.times do
      spawn_enemy(ENEMY_BOSS)
    end

    @broken_doors = []
    @explosions = []

    @font = RES::Vera
    @font.config(color: Colors::YELLOW)

    @physics.simulation = true
    @time = 0

    RES::Music.music()
  end

  DIR_KEYS = {
    KEY_W => [0, -1],
    KEY_S => [0, 1],
    KEY_A => [-1, 0],
    KEY_D => [1, 0],
    KEY_UP => [0, -1],
    KEY_DOWN => [0, 1],
    KEY_LEFT => [-1, 0],
    KEY_RIGHT => [1, 0],
  }

  def update
    dx = Mouse[Mouse::X] - SCREENX / 2
    dy = Mouse[Mouse::Y] - SCREENY / 2
    #player.angle = Math.atan2(dy, dx)

    step = [0, 0]
    DIR_KEYS.each do |k, v|
      if Engine::Key[k] != :up
        step[0] += v[0]
        step[1] += v[1]
      end
    end
    player.target = vec2(100 * step[0], 100 * step[1])

    @broken_doors.uniq.each(&:open)
    @broken_doors = []

    @explosions.uniq.each(&:explode)
    @explosions = []

    @physics.update

    if @player.hp < @player.pain
      @gamma_timer += 1
      new_player if @gamma_timer > DEATH_TIME
    else
      @gamma_timer = 0
    end
    #spawn_enemy(false) if @objects.size < 100

    super
  end

  def draw
    # RES::Space.background
    camera(@player.x - SCREENX / 2, @player.y - SCREENY / 2)
    super
    Engine.layer = 100
    player.show_hud

    if @gamma_timer > 0
      Engine.rect2(0, 0, SCREENX, SCREENY, true, Engine::Colors.new(55, 0, 0, 100 + 155 * (@gamma_timer / DEATH_TIME)))
    end

    v = 19 * 60 - @time.to_i
    min = (v / 60).to_s.rjust(2, "0")
    sec = (v % 60).to_s.rjust(2, "0")
    @font.draw_text("Remaining time: #{min}:#{sec}", 10, 10)
  end
end

app = FTPT.new
loop do
  app.update
  app.draw
  Engine::process
  $app.time += Engine[Params::DeltaTime] / 1000.0
  break if Engine::Key[QUIT] != :up || Engine::Key[KEY_ESCAPE] != :up
end
