require_relative "./lib/nonoengine"
require_relative "./lib/bearlibmg"
require_relative "./resources"
require_relative "./mapgen"
require_relative "./walls"

class Map < GameObject
  attr_reader :size

  def initialize
    @size = MAP_SIZE
    @map_data = []
    generate
  end

  def generate
    BearMapGenerator.new(Bearlib::G_NICE_DUNGEON, true).generate @map_data
    @walls = Walls.new(self)
  end

  def cell(x, y)
    @map_data[x + y * @size]
  end

  def random_point(typ)
    safety = 10000
    while safety > 0
      ax = rand(@size)
      ay = rand(@size)
      return [ax, ay] if cell(ax, ay)[:logic] == typ
      safety -= 1
    end
    puts "Safety check failed"
    return nil
  end

  def draw
    player_pos = [$app.player.mapx, $app.player.mapy]
    (-SIGHT_X..SIGHT_X).each do |px|
      (-SIGHT_Y..SIGHT_Y).each do |py|
        x = player_pos[0] + px
        next if x < 0 || x >= @size
        y = player_pos[1] + py
        next if y < 0 || y >= @size
        RES::All.draw_frame(cell(x, y)[:picture], x * TILESIZE + TILESIZE / 2, y * TILESIZE + TILESIZE / 2, TILESIZE / SPRITESIZE, TILESIZE / SPRITESIZE)
      end
    end
  end
end

class GameObject
  def mapx
    (x / TILESIZE).to_i
  end

  def mapy
    (y / TILESIZE).to_i
  end
end
