require "chipmunk"
require "./graph_consts"
require "./consts"
require "./lib/bearlibmg"
require "./lib/bearlibmap"

module TT
  FLOOR = 0
  WALL = 1
  NORTH = 2
  EAST = 3
  SOUTH = 4
  WEST = 5
  NE = 6
  SE = 7
  SW = 8
  NW = 9
  CORN_NE = 10
  CORN_SE = 11
  CORN_SW = 12
  CORN_NW = 13
  DOOR_N = 14
  DOOR_E = 15
  DOOR_S = 16
  DOOR_W = 17
end

module LogicalTile
  Wall = 0
  Room = 1
  Passage = 2
  VertDoor = 3
  HorizDoor = 4
  UpStair = 5
  DownStair = 6
  Chest = 7
end

def door?(x)
  [LogicalTile::VertDoor, LogicalTile::HorizDoor].include? x
end

class MapGenerator
end

class BearMapGenerator < MapGenerator
  def initialize(typ, double)
    @double = double
    @typ = typ
    @tile_data = []
    @size = @double ? MAP_SIZE / 2 : MAP_SIZE
  end

  def double_size
    new_tile_data = Array.new(@size * @size * 4) do |i|
      x = i % (@size * 2)
      y = i / (@size * 2)
      @tile_data[y / 2 * @size + x / 2]
    end
    @tile_data = new_tile_data
    @size *= 2
  end

  private def get_tile(north, south, west, east, nw, ne, sw, se)
    return TT::NW if north && west
    return TT::NE if north && east
    return TT::SW if south && west
    return TT::SE if south && east
    return TT::NORTH if north
    return TT::SOUTH if south
    return TT::WEST if west
    return TT::EAST if east
    return TT::CORN_NW if nw
    return TT::CORN_NE if ne
    return TT::CORN_SW if sw
    return TT::CORN_SE if se
    return TT::FLOOR
  end
  private def get_door(north, south, west, east)
    return TT::DOOR_N if north
    return TT::DOOR_S if south
    return TT::DOOR_W if west
    return TT::DOOR_E if east
    return TT::FLOOR
  end

  def get_logical(cur, tile)
    return LogicalTile::Wall if tile == TT::WALL
    return LogicalTile::VertDoor if [TT::DOOR_N, TT::DOOR_S].include? tile
    return LogicalTile::HorizDoor if [TT::DOOR_W, TT::DOOR_E].include? tile
    return LogicalTile::Room if cur == Bearlib::TILE_BIGROOM
    return LogicalTile::Passage
  end

  def generate(data)
    bearmap = Bearlib::Map.new(@size, @size, {cells: :integer})
    layer = bearmap.layers[:cells]
    layer.generate(@typ, SEED)
    @tile_data = Array.new(@size * @size)
    (@size * @size).times do |i|
      x = i % @size
      y = i / @size
      @tile_data[y * @size + x] = layer[x, y]
    end
    double_size if @double
    (@size * @size).times do |i|
      x = i % @size
      y = i / @size

      cur = @tile_data[y * @size + x]
      if !Bearlib.passable?(cur) || x == 0 || x == @size - 1 || y == 0 || y == @size - 1
        tile = TT::WALL
      else
        north = !Bearlib.passable?(@tile_data[x + (y - 1) * @size])
        east = !Bearlib.passable?(@tile_data[(x + 1) + y * @size])
        west = !Bearlib.passable?(@tile_data[(x - 1) + y * @size])
        south = !Bearlib.passable?(@tile_data[x + (y + 1) * @size])
        nw = !Bearlib.passable?(@tile_data[(x - 1) + (y - 1) * @size])
        ne = !Bearlib.passable?(@tile_data[(x + 1) + (y - 1) * @size])
        sw = !Bearlib.passable?(@tile_data[(x - 1) + (y + 1) * @size])
        se = !Bearlib.passable?(@tile_data[(x + 1) + (y + 1) * @size])

        door = false
        if cur == Bearlib::TILE_DOOR
          if north || south
            door = @tile_data[(x + 1) + y * @size] == Bearlib::TILE_BIGROOM || @tile_data[(x - 1) + y * @size] == Bearlib::TILE_BIGROOM
          else
            door = @tile_data[x + (y + 1) * @size] == Bearlib::TILE_BIGROOM || @tile_data[x + (y - 1) * @size] == Bearlib::TILE_BIGROOM
          end
        end

        if door
          tile = get_door(north, south, west, east)
        else
          tile = get_tile(north, south, west, east, nw, ne, sw, se)
        end
      end
      data[i] = {
        pass: Bearlib.passable?(cur), #&& cur != Bearlib::TILE_DOOR,
        picture: BACK_TILES[tile.to_i],
        logic: get_logical(cur, tile),
      }
    end
  end
end
