require_relative "consts"

class PhysicObject < GameObject
  attr_accessor :body

  def initialize(material)
    super()
    @body = physics.addbody self, material
    @shape = init_form
    physics.fit_shape @shape, self.class, material
  end

  def physics
    $app.physics
  end

  def init_form
    physics.add_circle_shape @body, OBJSIZE / 2, 0, 0
  end

  def warp(x, y)
    @body.p = vec2(x, y)
  end

  def x
    @body.p.x
  end

  def y
    @body.p.y
  end

  def pos
    @body.p
  end

  def vx
    @body.v.x
  end

  def vy
    @body.v.y
  end

  def speed
    @body.v
  end

  def die
    physics.del_body(@body)
  end
end

class ControllableObject < PhysicObject
  attr_accessor :target

  def initialize(material)
    super
    @control_body = CP::Body.new_static
    @control_body.p = @body.p
    @target = CP::ZERO_VEC_2
    @leash = CP::Constraint::PivotJoint.new(@control_body, @body, CP::ZERO_VEC_2, CP::ZERO_VEC_2)
    @leash.max_bias = material[:leash_speed]
    @leash.max_force = material[:leash_force]
    physics.add_constraint(@leash)
  end

  def warp(x, y)
    super
    @target = CP::ZERO_VEC_2
    @control_body.p = @body.p
  end

  def update
    super
    @control_body.p = @body.p + @target
  end
end
