require "./skills/skills"
require_relative "./lib/nonoengine"

# Our Player
class Player < ControllableObject
  attr_accessor :hp, :pain, :left_skill, :right_skill, :passive

  def initialize(klass)
    super(klass[:material])
    @klass = klass
    @picture = klass[:picture]
    @at_door = nil
    @font = RES::Vera
    @font.config(color: Colors::WHITE)
    @left_skill = klass[:left_skill][:class].new(self, klass[:left_skill])
    @right_skill = klass[:right_skill][:class].new(self, klass[:right_skill])
    @passive = klass[:passive][:class].new(self, klass[:passive]) if klass[:passive]
    @hp = PLAYER_HP
    @pain = 0.0
  end

  def update
    super
    @passive.update if @passive
    @pain -= PAIN_DECREMENT if @pain > 0
    @at_door = nil if @at_door && @at_door.broken
    @at_door = nil if @at_door && (mapx - @at_door.x).abs + (mapy - @at_door.y).abs > 1

    dx = Mouse[Mouse::X] - SCREENX / 2
    dy = Mouse[Mouse::Y] - SCREENY / 2
    @left_skill.update
    @right_skill.update
    if Mouse[Mouse::LEFT_BUTTON] != :up
      @left_skill.apply(dx, dy, Mouse[Mouse::LEFT_BUTTON])
    end
    if Mouse[Mouse::RIGHT_BUTTON] != :up
      @right_skill.apply(dx, dy, Mouse[Mouse::RIGHT_BUTTON])
    end

    if @at_door && Engine::Key[KEY_SPACE] != :up
      @at_door.open
      @at_door = nil
    end
  end

  def side
    SIDE_ALLY
  end

  def draw
    RES::All.draw_frame(@picture, x, y, OBJSIZE / SPRITESIZE, OBJSIZE / SPRITESIZE)
    #RES::Slash.draw_frame(1, @control_body.p.x, @control_body.p.y, OBJSIZE / SPRITESIZE, OBJSIZE / SPRITESIZE)
  end

  def take_hit(bullet)
    @hp -= bullet.stats[:damage] * DAM_WOUNDS
    @pain += bullet.stats[:damage] * DAM_PAIN
  end

  def show_hud
    @font.draw_text(@at_door.message, SCREENX / 2 - 100, SCREENY / 2 - 100) if @at_door
    show_progress(100, SCREENY - 100, 50, @left_skill.progress)
    show_progress(SCREENX - 100, SCREENY - 100, 50, @right_skill.progress)

    unless @passive.class == NoPain
      w = 450
      damaged = w * (PLAYER_HP - @hp) / PLAYER_HP
      pains = w * @pain / PLAYER_HP
      basex = SCREENX / 2 - w / 2
      Engine.rect2(basex, SCREENY - 50, w, 25, true, Engine::Colors.new(0, 50, 0, 50))
      Engine.rect2(basex, SCREENY - 50, damaged, 25, true, Engine::Colors.new(0, 0, 0))
      Engine.rect2(basex + damaged, SCREENY - 50, pains, 25, true, Engine::Colors.new(0xff, 0, 0))
    end
  end

  def collide_door(door)
    @at_door = door
    true
  end
end
