# encoding: utf-8

class Door
  attr_accessor :shape, :x, :y, :broken

  def initialize(ax, ay)
    @broken = false
    @x = ax
    @y = ay
  end

  def message
    "Нажмите Пробел чтобы открыть"
  end

  def break
    @broken = true
    $app.broken_doors << self
  end

  def open
    @broken = true
    cell = $app.map.cell(@x, @y)
    cell[:pass] = true
    cell[:logic] = LogicalTile::Passage
    cell[:picture] = OPEN_DOORS[cell[:picture]]
    $app.physics.del_shape @shape
  end
end

class Walls
  attr_reader :map

  def poly_shape(v)
    shape = @physics.add_poly_shape(@body, v)
    @physics.fit_shape(shape, Walls, MAT_WALL)
    shape
  end

  BORDER = 25

  def add_tile(x, y, scale)
    x1 = x * scale - BORDER
    y1 = y * scale - BORDER
    x2 = x1 + scale + BORDER
    y2 = y1 + scale + BORDER
    v = [vec2(x1, y1), vec2(x1, y2), vec2(x2, y2), vec2(x2, y1)]
    poly_shape(v)
  end

  DOOR_WIDTH2 = 0.1

  def add_door(x, y, vertical, scale, door_obj)
    if vertical
      x1 = (x + 0.5) * scale - DOOR_WIDTH2
      y1 = y * scale - BORDER
      x2 = (x + 0.5) * scale + DOOR_WIDTH2
      y2 = y1 + scale + BORDER
    else
      x1 = x * scale - BORDER
      y1 = (y + 0.5) * scale - DOOR_WIDTH2
      x2 = x1 + scale + BORDER
      y2 = (y + 0.5) * scale + DOOR_WIDTH2
    end
    v = [vec2(x1, y1), vec2(x1, y2), vec2(x2, y2), vec2(x2, y1)]
    sh = poly_shape(v)
    sh.object = door_obj
    sh.collision_type = Door
    sh
  end

  def initialize(amap)
    @physics = $app.physics
    @body = @physics.add_static_body(self)
    @map = amap

    reset
    @map.size.times do |x|
      @map.size.times do |y|
        case @map.cell(x, y)[:logic]
        when LogicalTile::VertDoor
          door = Door.new(x, y)
          door.shape = add_door(x, y, true, TILESIZE, door)
        when LogicalTile::HorizDoor
          door = Door.new(x, y)
          door.shape = add_door(x, y, false, TILESIZE, door)
        when LogicalTile::Wall
          add_tile(x, y, TILESIZE)
        else
          #
        end
      end
    end
  end

  def reset
    @physics.clear_body @body
  end
end
